@@ -43,6 +43,7 @@ import getpass
 import signal
 import ctypes
 import string
+import glob
 
 try:
     import requests
@@ -93,10 +94,18 @@ try:
         sf.set_option("VariantPath", "variants.ini")
     except Exception:
         print("No variants.ini found.", file=sys.stderr)
+        raise
+
+    try:
+        print(sf.version())
+    except Exception:
+        print("fairyfishnet requires pyffish >=0.0.51", file=sys.stderr)
+        raise
 
 except ImportError:
     print("No pyffish module installed!", file=sys.stderr)
     sf_ok = False
+    raise
 
 try:
     # Python 3
@@ -106,7 +115,7 @@ except NameError:
     DEAD_ENGINE_ERRORS = (EOFError, IOError)
 
 
-__version__ = "1.15.44"
+__version__ = "1.15.50"
 
 __author__ = "Bajusz Tamás"
 __email__ = "gbtami@gmail.com"
@@ -472,16 +481,8 @@ def go(p, position, moves, movetime=None, clock=None, depth=None, nodes=None, va
             bestmove = arg.split()[0]
             if bestmove and bestmove != "(none)":
                 info["bestmove"] = bestmove
-
-            # Convert PV move list to SAN using pyffish
-            if sf_ok and nodes is not None and "pv" in info and info["pv"]:
-                try:
-                    fen = sf.get_fen(variant, position, moves, chess960)
-                    notation = sf.NOTATION_JANGGI if variant == "janggi" else sf.NOTATION_DEFAULT
-                    info["pv_san"] = " ".join(sf.get_san_moves(variant, fen, info["pv"].split(), chess960, notation))
-                except Exception:
-                    logging.error("Failed converting PV moves to SAN")
             return info
+
         elif command == "info":
             arg = arg or ""
 
@@ -549,6 +550,10 @@ def set_variant_options(p, variant, chess960):
 
     setoption(p, "UCI_Chess960", chess960)
 
+    eval_file = variant + ".nnue"
+    if os.path.isfile(eval_file):
+        setoption(p, "EvalFile", eval_file)
+
     if variant in ["standard", "fromposition", "chess960"]:
         setoption(p, "UCI_Variant", "chess")
     else:
@@ -781,6 +786,9 @@ class Worker(threading.Thread):
             for name, value in self.conf.items("Stockfish"):
                 self.stockfish_info["options"][name] = value
 
+        # Add .nnue file list
+        self.stockfish_info["nnue"] = glob.glob("*.nnue")
+
         # Set UCI options
         for name, value in self.stockfish_info["options"].items():
             setoption(self.stockfish, name, value)
@@ -1052,6 +1060,9 @@ def download_github_release(conf, release_page, filename):
         sys.stderr.write("\n")
         sys.stderr.flush()
 
+    if filename.endswith(".nnue"):
+        return filename
+
     # Make executable
     logging.info("chmod +x %s", filename)
     st = os.stat(path)
@@ -1544,6 +1555,9 @@ def cmd_run(args):
         print()
         stockfish_command = get_stockfish_command(conf)
 
+    # Check .nnue updates (only for shogi at this moment)
+    download_github_release(conf, STOCKFISH_RELEASES, "shogi.nnue")
+
     print()
     print("### Checking configuration ...")
     print()
@@ -1915,63 +1929,120 @@ def create_variants_ini(args):
     engine_dir = get_engine_dir(conf)
 
     ini_text = textwrap.dedent("""\
-    # Hybrid variant of Grand-chess and crazyhouse, using Grand-chess as a template
-    [grandhouse:grand]
-    startFen = r8r/1nbqkcabn1/pppppppppp/10/10/10/10/PPPPPPPPPP/1NBQKCABN1/R8R[] w - - 0 1
-    pieceDrops = true
-    capturesToHand = true
+# Hybrid variant of Grand-chess and crazyhouse, using Grand-chess as a template
+[grandhouse:grand]
+startFen = r8r/1nbqkcabn1/pppppppppp/10/10/10/10/PPPPPPPPPP/1NBQKCABN1/R8R[] w - - 0 1
+pieceDrops = true
+capturesToHand = true
 
-    # Hybrid variant of Gothic-chess and crazyhouse, using Capablanca as a template
-    [gothhouse:capablanca]
-    startFen = rnbqckabnr/pppppppppp/10/10/10/10/PPPPPPPPPP/RNBQCKABNR[] w KQkq - 0 1
-    pieceDrops = true
-    capturesToHand = true
+# Hybrid variant of Gothic-chess and crazyhouse, using Capablanca as a template
+[gothhouse:capablanca]
+startFen = rnbqckabnr/pppppppppp/10/10/10/10/PPPPPPPPPP/RNBQCKABNR[] w KQkq - 0 1
+pieceDrops = true
+capturesToHand = true
 
-    # Shogun chess
-    [shogun:crazyhouse]
-    startFen = rnb+fkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNB+FKBNR w KQkq - 0 1
-    commoner = c
-    centaur = g
-    archbishop = a
-    chancellor = m
-    fers = f
-    promotionRank = 6
-    promotionLimit = g:1 a:1 m:1 q:1
-    promotionPieceTypes = -
-    promotedPieceType = p:c n:g b:a r:m f:q
-    mandatoryPawnPromotion = false
-    firstRankPawnDrops = true
-    promotionZonePawnDrops = true
-    whiteDropRegion = *1 *2 *3 *4 *5
-    blackDropRegion = *4 *5 *6 *7 *8
-    immobilityIllegal = true
+[shogun:crazyhouse]
+variantTemplate = shogi
+pieceToCharTable = PNBR.F.....++++.+Kpnbr.f.....++++.+k
+pocketSize = 8
+startFen = rnb+fkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNB+FKBNR[] w KQkq - 0 1
+commoner = c
+centaur = g
+archbishop = a
+chancellor = m
+fers = f
+promotionRank = 6
+promotionLimit = g:1 a:1 m:1 q:1
+promotionPieceTypes = -
+promotedPieceType = p:c n:g b:a r:m f:q
+mandatoryPawnPromotion = false
+firstRankPawnDrops = true
+promotionZonePawnDrops = true
+whiteDropRegion = *1 *2 *3 *4 *5
+blackDropRegion = *4 *5 *6 *7 *8
+immobilityIllegal = true
 
-    # Asymmetric variant with one army using pieces that move like knights but attack like other pieces (kniroo and knibis)
-    [orda:chess]
-    startFen = lhaykahl/8/pppppppp/8/8/8/PPPPPPPP/RNBQKBNR w KQ - 0 1
-    centaur = h
-    knibis = a
-    kniroo = l
-    silver = y
-    promotionPieceTypes = qh
-    flagPiece = k
-    whiteFlag = *8
-    blackFlag = *1
+[orda:chess]
+pieceToCharTable = PNBRQ..AH...........LKp...q..ah.y.........lk
+centaur = h
+knibis = a
+kniroo = l
+silver = y
+promotionPieceTypes = qh
+startFen = lhaykahl/8/pppppppp/8/8/8/PPPPPPPP/RNBQKBNR w KQ - 0 1
+flagPiece = k
+whiteFlag = *8
+blackFlag = *1
 
-    [synochess:pocketknight]
-    janggiCannon = c
-    soldier = s
-    horse = h
-    fersAlfil = e
-    commoner = a
-    startFen = rneakenr/8/1c4c1/1ss2ss1/8/8/PPPPPPPP/RNBQKBNR[ss] w KQ - 0 1
-    stalemateValue = loss
-    perpetualCheckIllegal = true
-    flyingGeneral = true
-    blackDropRegion = *5
-    flagPiece = k
-    whiteFlag = *8
-    blackFlag = *1""")
+[synochess:pocketknight]
+pieceToCharTable = PNBRQAE...SCH........Kpnbrqae...sch........k
+pocketSize = 8
+janggiCannon = c
+soldier = s
+horse = h
+fersAlfil = e
+commoner = a
+startFen = rneakenr/8/1c4c1/1ss2ss1/8/8/PPPPPPPP/RNBQKBNR[ss] w KQ - 0 1
+stalemateValue = loss
+perpetualCheckIllegal = true
+flyingGeneral = true
+blackDropRegion = *5
+flagPiece = k
+whiteFlag = *8
+blackFlag = *1
+
+[shinobi:crazyhouse]
+variantTemplate = shogi
+pieceToCharTable = PNBRQ.DJMLH.....CKpnbrq.djmlh.....ck
+pocketSize = 8
+commoner = c
+bers = d
+archbishop = j
+fers = m
+shogiKnight = h
+lance = l
+promotionRank = 7
+promotionPieceTypes = -
+promotedPieceType = p:c m:b h:n l:r
+mandatoryPiecePromotion = true
+stalemateValue = loss
+perpetualCheckIllegal = true
+startFen = rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/LH1CK1HL[LHMMDJ] w kq - 0 1
+capturesToHand = false
+whiteDropRegion = *1 *2 *3 *4
+immobilityIllegal = true
+flagPiece = k
+whiteFlag = *8
+blackFlag = *1
+
+[ordamirror:chess]
+pieceToCharTable = P...Q..AH.F.........LKp...q..ah.f.........lk
+centaur = h
+knibis = a
+kniroo = l
+customPiece1 = f:mQcN
+promotionPieceTypes = qh
+startFen = lhafkahl/8/pppppppp/8/8/PPPPPPPP/8/LHAFKAHL w - - 0 1
+flagPiece = k
+whiteFlag = *8
+blackFlag = *1
+
+[empire:chess]
+pieceToCharTable = PNBRQ.....ST.C.D.E...Kpnbrq.....st.c.d.e...k
+customPiece1 = e:mQcN
+customPiece2 = c:mQcB
+customPiece3 = t:mQcR
+customPiece4 = d:mQcK
+soldier = s
+promotionPieceTypes = qne
+startFen = rnbqkbnr/pppppppp/8/8/8/PPPSSPPP/8/TECDKCET w kq - 0 1
+stalemateValue = loss
+nFoldValue = loss
+flagPiece = k
+whiteFlag = *8
+blackFlag = *1
+flyingGeneral = true
+""")
 
     ini_file = os.path.join(engine_dir, "variants.ini")
     print(ini_text, file=open(ini_file, "w"))
